#!/usr/bin/perl -w
##############################################################################
# file: Mascot Daemon Export Extender                                        #
# A perl script to extend the export functionality of Mascot Daemon          #
##############################################################################
# COPYRIGHT NOTICE                                                           #
# Copyright 1998-2023 Matrix Science Limited  All Rights Reserved.           #
#                                                                            #
##############################################################################
#     $Source: MDXE.pl,v $
#     $Author: richardj $
#       $Date: Wed Mar 22 16:55:31 2023 $
#   $Revision: 1.0 $ 
##############################################################################
use strict;
use Getopt::Std;
use Text::ParseWords;
use File::Basename;
use File::Spec;

##############################################################################

# Command line inputs
my (
    %opts,
    $configFilename,
    $inputDir,
    $loggingFlag,
    $logfile,
);
$loggingFlag = 0;

# Global logging handle
my $LOG;

getopts("hc:i:l", \%opts);
usage() if $opts{'h'};

if (defined $opts{'c'}){
    $configFilename = $opts{'c'};
}

# Input directory where results files reside; typically the Mascot
# Daemon task directory <task_directory>
if (defined $opts{'i'}){
    $inputDir = $opts{'i'};
}

if (not $configFilename or not $inputDir) {
    usage();
}

if (defined $opts{'l'}){
    $loggingFlag = 1;
    $logfile = $inputDir . "\\MDXE.log";
    open($LOG, '>', $logfile) or die "Could not open '$logfile' for writing: $!";
    printLog("-=-=-=-=-=-=-=Start up-=-=-=-=-=-=-=");
    printLog("Config file is '$configFilename'");
    printLog("Starting directory is '$inputDir'");
}


my (
    %config,
    @files,
);

open (my $INPUT, '<', $configFilename) or die "Could not open '$configFilename' for reading: $!";
while (<$INPUT>){
    chomp;
    # Skip comment lines
    if (/^\s*\#/){
        next;
    }

    # Skip empty lines
    if (/^\s*$/) {
        next;
    }

    my ($key, $value) = split /=/, $_;
    $config{$key} = $value;
}
close $INPUT;

# Read in the files from the input directory
do {
    opendir my $dh, $inputDir or die "Could not open '$inputDir' for reading: $!";
    @files = readdir $dh;
    closedir $dh;
};

# Sort the config values so that we run command1 before command2
my @configKeys = sort { $a cmp $b } keys %config;
foreach my $key (@configKeys){
    if ($key =~ /command\d+/) {
        # This is a command line
        if ($loggingFlag == 1) {
            printLog("Preparing $key");
        }
        # Parse line to find out which variables to use
        # Split command line into tokens
        my @command = quotewords('\<|\>', 1, $config{$key});    

        # Determine input and output types 
        my ($inputType, $outputType);
        foreach my $token (@command) {
            if ($token=~ /input\d+/){
                $inputType = $config{$token};        
                next;
            } elsif ($token =~ /output\d+/){
                $outputType = $config{$token};        
                next;
            }
        }
        # Find files that match input filter
        foreach my $currentFile (@files) {
            my $extension = (fileparse($currentFile, qr/\.[^.]*/))[2];
            if ($extension eq $inputType){
                # This is a file we want
                # Create output file
                my $outputFilename = $currentFile . $outputType;
                # Assemble commandline
                if ($loggingFlag == 1) {
                    printLog("Processing file '$currentFile'");
                    printLog("Output file '$outputFilename'");
                }
                my $commandLine = '';
                foreach my $token (@command) {
                    if ($token =~ /exec\d+/){
                        $commandLine .= "\"" . $config{$token} . "\"";        
                        next;
                    }
                    elsif ($token =~ /input\d+/){
                        $commandLine .= "\"" . $inputDir . "\\" . $currentFile . "\"";    
                        next;
                    }
                    elsif ($token =~ /output\d+/){
                        $commandLine .= "\"" . $inputDir . "\\" . $outputFilename . "\"";        
                        next;
                    }
                    elsif ($token =~ /taskDir\d+/){
                        $commandLine .= "\"" . $inputDir . "\"";        
                        next;
                    }                    
                    else {
                        $commandLine .= $token;        
                        next;
                    }
                }
                if ($loggingFlag == 1) {
                    printLog("Assembled command line '$commandLine'");
                }
                # Run the command
                my $commandLineOut = qx/$commandLine/;
                if ($loggingFlag == 1) {
                    printLog ("Command line output: $commandLineOut");
                    printLog ("Command line completed\n");
                }
            }
            else {
                next;
            }
        }    
    }
}

if ($loggingFlag == 1) {
    printLog("Run complete\n");
    close $LOG;
}

#######################
# Print to log Subroutine
# Richard Jacob
#
#######################
sub printLog {
  print $LOG (scalar localtime() . " @_\n");
}

#######################
# Usage Subroutine
# Richard Jacob
#
#######################
sub usage {
    my $abspath = File::Spec->rel2abs($0);

print STDERR << "EOF";
MDXE.pl - Mascot Daemon Export Extender

usage: $0 -i <task_directory> -c path\\to\\config.txt

    -h                      : this (help) message
    -c config filename      : Path to config filename
    -i input directory      : Data directory, Mascot Daemon tag <task_directory>
    -l                      : Turn on logging

Command line example:  $0 -c "c:/temp/MDXEconfig.txt" -i .
Mascot Daemon example: perl "$abspath" -c "c:/temp/MDXEconfig.txt" -i <task_directory>

EOF
    exit;
}

